@@ -1,24 +1,36 @@
 import os
+import shutil
 from datetime import datetime
+from unittest import mock
 
 import mongomock.collection
+from monty.tempfile import ScratchDir
 import pymongo.collection
 import pytest
 from pymongo.errors import ConfigurationError, DocumentTooLarge, OperationFailure
 
+import maggma.stores
 from maggma.core import StoreError
 from maggma.stores import JSONStore, MemoryStore, MongoStore, MongoURIStore
+from maggma.stores.mongolike import MontyStore
 from maggma.validators import JSONSchemaValidator
 
 
 @pytest.fixture
 def mongostore():
-    store = MongoStore("maggma_test", "test")
+    store = MongoStore(database="maggma_test", collection_name="test",)
     store.connect()
     yield store
     store._collection.drop()
 
 
+@pytest.fixture
+def montystore(tmp_dir):
+    store = MontyStore("maggma_test")
+    store.connect()
+    return store
+
+
 @pytest.fixture
 def memorystore():
     store = MemoryStore()
@@ -36,11 +48,20 @@ def jsonstore(test_dir):
     return JSONStore(files)
 
 
-def test_mongostore_connect():
+@pytest.mark.xfail(raises=StoreError)
+def test_mongostore_connect_error():
     mongostore = MongoStore("maggma_test", "test")
-    assert mongostore._collection is None
+    mongostore.count()
+
+
+def test_mongostore_connect_reconnect():
+    mongostore = MongoStore("maggma_test", "test")
+    assert mongostore._coll is None
     mongostore.connect()
     assert isinstance(mongostore._collection, pymongo.collection.Collection)
+    mongostore.close()
+    assert mongostore._coll is None
+    mongostore.connect()
 
 
 def test_mongostore_query(mongostore):
@@ -164,7 +185,7 @@ def test_mongostore_from_db_file(mongostore, db_json):
 
 
 def test_mongostore_from_launchpad_file(lp_file):
-    ms = MongoStore.from_launchpad_file(lp_file, collection_name='tmp')
+    ms = MongoStore.from_launchpad_file(lp_file, collection_name="tmp")
     ms.connect()
     assert ms._collection.full_name == "maggma_tests.tmp"
 
@@ -174,7 +195,7 @@ def test_mongostore_from_collection(mongostore, db_json):
     ms.connect()
 
     other_ms = MongoStore.from_collection(ms._collection)
-    assert ms._collection.full_name == other_ms._collection.full_name
+    assert ms._coll.full_name == other_ms._collection.full_name
     assert ms.database == other_ms.database
 
 
@@ -231,7 +252,7 @@ def test_mongostore_newer_in(mongostore):
 # Memory store tests
 def test_memory_store_connect():
     memorystore = MemoryStore()
-    assert memorystore._collection is None
+    assert memorystore._coll is None
     memorystore.connect()
     assert isinstance(memorystore._collection, mongomock.collection.Collection)
 
@@ -269,6 +290,116 @@ def test_groupby(memorystore):
     assert len(data) == 2
 
 
+# Monty store tests
+def test_monty_store_connect(tmp_dir):
+    montystore = MontyStore(collection_name="my_collection")
+    assert montystore._coll is None
+    montystore.connect()
+    assert montystore._collection is not None
+
+
+def test_monty_store_groupby(montystore):
+    montystore.update(
+        [
+            {"e": 7, "d": 9, "f": 9},
+            {"e": 7, "d": 9, "f": 10},
+            {"e": 8, "d": 9, "f": 11},
+            {"e": 9, "d": 10, "f": 12},
+        ],
+        key="f",
+    )
+    data = list(montystore.groupby("d"))
+    assert len(data) == 2
+    grouped_by_9 = [g[1] for g in data if g[0]["d"] == 9][0]
+    assert len(grouped_by_9) == 3
+    grouped_by_10 = [g[1] for g in data if g[0]["d"] == 10][0]
+    assert len(grouped_by_10) == 1
+
+    data = list(montystore.groupby(["e", "d"]))
+    assert len(data) == 3
+
+    montystore.update(
+        [
+            {"e": {"d": 9}, "f": 9},
+            {"e": {"d": 9}, "f": 10},
+            {"e": {"d": 9}, "f": 11},
+            {"e": {"d": 10}, "f": 12},
+        ],
+        key="f",
+    )
+    data = list(montystore.groupby("e.d"))
+    assert len(data) == 2
+
+
+def test_montystore_query(montystore):
+    montystore._collection.insert_one({"a": 1, "b": 2, "c": 3})
+    assert montystore.query_one(properties=["a"])["a"] == 1
+    assert montystore.query_one(properties=["a"])["a"] == 1
+    assert montystore.query_one(properties=["b"])["b"] == 2
+    assert montystore.query_one(properties=["c"])["c"] == 3
+
+
+def test_montystore_count(montystore):
+    montystore._collection.insert_one({"a": 1, "b": 2, "c": 3})
+    assert montystore.count() == 1
+    montystore._collection.insert_one({"aa": 1, "b": 2, "c": 3})
+    assert montystore.count() == 2
+    assert montystore.count({"a": 1}) == 1
+
+
+def test_montystore_distinct(montystore):
+    montystore._collection.insert_one({"a": 1, "b": 2, "c": 3})
+    montystore._collection.insert_one({"a": 4, "d": 5, "e": 6, "g": {"h": 1}})
+    assert set(montystore.distinct("a")) == {1, 4}
+
+    # Test list distinct functionality
+    montystore._collection.insert_one({"a": 4, "d": 6, "e": 7})
+    montystore._collection.insert_one({"a": 4, "d": 6, "g": {"h": 2}})
+
+    # Test distinct subdocument functionality
+    ghs = montystore.distinct("g.h")
+    assert set(ghs) == {1, 2}
+
+    # Test when key doesn't exist
+    assert montystore.distinct("blue") == []
+
+    # Test when null is a value
+    montystore._collection.insert_one({"i": None})
+    assert montystore.distinct("i") == [None]
+
+
+def test_montystore_update(montystore):
+    montystore.update({"e": 6, "d": 4}, key="e")
+    assert (
+        montystore.query_one(criteria={"d": {"$exists": 1}}, properties=["d"])["d"] == 4
+    )
+
+    montystore.update([{"e": 7, "d": 8, "f": 9}], key=["d", "f"])
+    assert montystore.query_one(criteria={"d": 8, "f": 9}, properties=["e"])["e"] == 7
+
+    montystore.update([{"e": 11, "d": 8, "f": 9}], key=["d", "f"])
+    assert montystore.query_one(criteria={"d": 8, "f": 9}, properties=["e"])["e"] == 11
+
+    test_schema = {
+        "type": "object",
+        "properties": {"e": {"type": "integer"}},
+        "required": ["e"],
+    }
+    montystore.validator = JSONSchemaValidator(schema=test_schema)
+    montystore.update({"e": 100, "d": 3}, key="e")
+
+    # Continue to update doc when validator is not set to strict mode
+    montystore.update({"e": "abc", "d": 3}, key="e")
+
+
+def test_montystore_remove_docs(montystore):
+    montystore._collection.insert_one({"a": 1, "b": 2, "c": 3})
+    montystore._collection.insert_one({"a": 4, "d": 5, "e": 6, "g": {"h": 1}})
+    montystore.remove_docs({"a": 1})
+    assert len(list(montystore.query({"a": 4}))) == 1
+    assert len(list(montystore.query({"a": 1}))) == 0
+
+
 def test_json_store_load(jsonstore, test_dir):
     jsonstore.connect()
     assert len(list(jsonstore.query())) == 20
@@ -278,6 +409,45 @@ def test_json_store_load(jsonstore, test_dir):
     assert len(list(jsonstore.query())) == 20
 
 
+def test_json_store_writeable(test_dir):
+    with ScratchDir("."):
+        shutil.copy(test_dir / "test_set" / "d.json", ".")
+        jsonstore = JSONStore("d.json", file_writable=True)
+        jsonstore.connect()
+        assert jsonstore.count() == 2
+        jsonstore.update({"new": "hello", "task_id": 2})
+        assert jsonstore.count() == 3
+        jsonstore.close()
+        jsonstore = JSONStore("d.json", file_writable=True)
+        jsonstore.connect()
+        assert jsonstore.count() == 3
+        jsonstore.remove_docs({"a": 5})
+        assert jsonstore.count() == 2
+        jsonstore.close()
+        jsonstore = JSONStore("d.json", file_writable=True)
+        jsonstore.connect()
+        assert jsonstore.count() == 2
+        jsonstore.close()
+        with mock.patch(
+            "maggma.stores.JSONStore.update_json_file"
+        ) as update_json_file_mock:
+            jsonstore = JSONStore("d.json", file_writable=False)
+            jsonstore.connect()
+            jsonstore.update({"new": "hello", "task_id": 5})
+            assert jsonstore.count() == 3
+            jsonstore.close()
+            update_json_file_mock.assert_not_called()
+        with mock.patch(
+            "maggma.stores.JSONStore.update_json_file"
+        ) as update_json_file_mock:
+            jsonstore = JSONStore("d.json", file_writable=False)
+            jsonstore.connect()
+            jsonstore.remove_docs({"task_id": 5})
+            assert jsonstore.count() == 2
+            jsonstore.close()
+            update_json_file_mock.assert_not_called()
+
+
 def test_eq(mongostore, memorystore, jsonstore):
     assert mongostore == mongostore
     assert memorystore == memorystore
